<?php
/**
 * Plugin Name:       SlicPix Interactivity Studio
 * Plugin URI:        https://interactivity.studio/
 * Description:       Create and embed interactive images and image maps on your WordPress site using Interactivity Studio. Increase user engagement and product conversions.
 * Version:           1.0.0
 * Requires at least: 6.1
 * Requires PHP:      7.0
 * Author:            Slicpix Inc.
 * Author URI:		  https://slicpix.com
 * License:			  GPL-2.0-or-later
 * License URI:		  https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       slicpix-interactivity-studio
 * 
 * @package SlicpixInteractivityStudio
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Registers the block using the metadata loaded from the `block.json` file.
 * Behind the scenes, it registers also all assets so they can be enqueued
 * through the block editor in the corresponding context.
 *
 * @see https://developer.wordpress.org/reference/functions/register_block_type/
 */
function slicpix_interactivity_studio_interactivity_studio_block_init() {
	register_block_type( __DIR__ . '/build' );
}
add_action( 'init', 'slicpix_interactivity_studio_interactivity_studio_block_init' );

/*****************************************************************
*
*                        REST API EXTENSIONS
*
*****************************************************************/

/**
 * Retrieves the Interactivity Studio's account key in the WP Options table.
 *
 * @since      1.0.0
 * @access     private
 *
 */
function slicpix_interactivity_studio_get_key($data) {
	$key = get_option("slicpix_interactivity_studio_key");
	if ($key == false ){
		$key="";
	}
	return rest_ensure_response($key);
};

/**
 * Persists the Interactivity Studio's account key in the WP Options table.
 *
 * @since      1.0.0
 * @access     private
 *
 */
function slicpix_interactivity_studio_persist_key($request) {
	$key = get_option("slicpix_interactivity_studio_key");
	if ($key == false ){
		$params = $request->get_json_params();
		$key = $params['key'];
		$added=add_option(
			"slicpix_interactivity_studio_key", 
			$key
		);
		if (!$added){
			$key="";
		}else{
			$key = get_option("slicpix_interactivity_studio_key");
		}
	}
	return rest_ensure_response($key);
};

/**
 * Delete the Interactivity Studio's account key from the WP Options table.
 *
 * @since      1.0.0
 * @access     private
 *
 */
function slicpix_interactivity_studio_delete_key($data) {
	delete_option("slicpix_interactivity_studio_key");
	$key = get_option("slicpix_interactivity_studio_key");
	if ($key == false ){
		$key="";
	}
	return rest_ensure_response($key);
};

/**
 * Persists an Interactivity Studio's image in the WP Media Gallery.
 *
 * @since      1.0.0
 * @access     private
 *
 */
function slicpix_interactivity_studio_upload_file_by_url($request){

	$params = $request->get_json_params();
	$image_url = $params['url'];

	$attachment_id = "";

	// it allows us to use download_url() and wp_handle_sideload() functions
	require_once( ABSPATH . 'wp-admin/includes/file.php' );

	// download to temp dir
	$temp_file = download_url( $image_url );

	if( is_wp_error( $temp_file ) ) {
		return rest_ensure_response($attachment_id);
	}

	// move the temp file into the uploads directory
	$file = array(
		'name'     => basename( $image_url ),
		'type'     => mime_content_type( $temp_file ),
		'tmp_name' => $temp_file,
		'size'     => filesize( $temp_file ),
	);

	$sideload = wp_handle_sideload(
		$file,
		array(
			'test_form'   => false // no needs to check 'action' parameter
		)
	);

	// it is time to add our uploaded image into WordPress media library
	$attachment_id = wp_insert_attachment(
		array(
			'guid'           => $sideload[ 'url' ],
			'post_mime_type' => $sideload[ 'type' ],
			'post_title'     => basename( $sideload[ 'file' ] ),
			'post_content'   => '',
			'post_status'    => 'inherit',
		),
		$sideload[ 'file' ]
	);

	if( is_wp_error( $attachment_id ) || ! $attachment_id ) {
		$attachment_id = "";
		return rest_ensure_response($attachment_id);
	}

	// update medatata, regenerate image sizes
	require_once( ABSPATH . 'wp-admin/includes/image.php' );

	wp_update_attachment_metadata(
		$attachment_id,
		wp_generate_attachment_metadata( $attachment_id, $sideload[ 'file' ] )
	);

	return rest_ensure_response($attachment_id);
}

/**
 * Retrieve the WP Media Gallery url of a Interactivity Studio's image.
 *
 * @since      1.0.0
 * @access     private
 *
 */
function slicpix_interactivity_studio_retrieve_image_url($request){
	$params = $request->get_json_params();
	$attachmentID = $params['attachment'];

	$img_src=wp_get_attachment_url( $attachmentID );

	if ( $img_src == false ){
		$img_src = "false";
		return rest_ensure_response($img_src);
	}

	return rest_ensure_response($img_src);
}

/**
 * Extend the WP Rest API with 5 additional routes.
 *
 * @since      1.0.0
 * @access     private
 *
 */
add_action('rest_api_init', function() {
	register_rest_route( 'slicpix_interactivity_studio', '/key', [
		'methods' => 'GET',
		'callback' => 'slicpix_interactivity_studio_get_key',
		'permission_callback' => function($request) {
            return current_user_can('upload_files');
		}
	]);
	register_rest_route( 'slicpix_interactivity_studio', '/key', [
		'methods' => 'POST',
		'callback' => 'slicpix_interactivity_studio_persist_key',
		'permission_callback' => function($request) {
			return current_user_can('manage_options');
		}
	]);
	register_rest_route( 'slicpix_interactivity_studio', '/key', [
		'methods' => 'DELETE',
		'callback' => 'slicpix_interactivity_studio_delete_key',
		'permission_callback' => function($request) {
			return current_user_can('manage_options');
		}
	]);

	register_rest_route( 'slicpix_interactivity_studio','/media_gallery',[
		'methods' => 'POST',
		'callback' => 'slicpix_interactivity_studio_upload_file_by_url',
		'permission_callback' => function($request) {
            return current_user_can('upload_files');
		}
	]);

	register_rest_route( 'slicpix_interactivity_studio','/media_gallery/image',[
		'methods' => 'POST',
		'callback' => 'slicpix_interactivity_studio_retrieve_image_url',
		'permission_callback' => function($request) {
            return current_user_can('upload_files');
		}
	]);
});

/**
 * Expand the WP wp_kses_allowed_html filter to allow additional tags and 
 * attributes.
 *
 * @since      1.0.0
 * @access     private
 *
 */
add_filter( 'wp_kses_allowed_html', function( $tags ){

		$tags['svg'] = array(
			'xmlns' => array(),
			'xmlns:xlink' => array(),
			'id' => array(),
			'viewbox' => array(),
			'class' => array(),
			'pointer-events' => array(),
			'tabindex' => array(),
			'role' => array(),
			'title' => array(),
			'aria-label' => array(),
			'width' => array(),
			'height' => array(),
			'style' => array(),
		);

		$tags['style'] = array(	
		);

		$tags['image'] = array(	
			'crossorigin' => array(),
			'width' => array(),
			'height' => array(),
			'visibility' => array(),
			'href' => array(),
		);

		$tags['g'] = array(
			'id' => array(),
			'class' => array(),
			'data-spix-id' => array(),
			'data-spix-name' => array(),
			'tabindex' => array(),
			'role' => array(),
			'aria-label' => array(),
			'text' => array(),
		);

		$tags['path'] = array(
			'data-spix-zone' => array(),
			'data-spix-name' => array(),
			'd' => array(),
			'data-spix-resource' => array(),
			'data-spix-type' => array(),
			'data-scope' => array(),
		);

		$tags['div'] = array(
			'class' => array(),
			'data-is-image' => array(),
			'style' => array(),
			'data-is-account' => array(),
			'data-is-third-party' => array(),
			'data-is-instanceid' => array(),
			'data-is-asset' => array(),
		);

		return $tags;

}, 10, 2);
